<?php

namespace QUI\ERP\Testing\Utils;

use QUI;
use QUI\ERP\Accounting\Payments\Payments;
use Doctrine\DBAL\Exception as DoctrineException;

use function array_rand;
use function str_starts_with;

class ErpEntities
{
    public static function getCustomer(): QUI\ERP\User
    {
        return new QUI\ERP\User([
            'id' => '',
            'uuid' => '',
            'username' => 'customer',
            'firstname' => 'Hans-Günther',
            'lastname' => 'Hamann',
            'address' => [
                'firstname' => 'Hans-Günther',
                'lastname' => 'Hamann',
                'street_no' => 'Philipp-Köhler-Weg 52a',
                'city' => 'Itzehoe',
                'zip' => '30638',
                'country' => 'DE',
            ],
            'lang' => 'de',
            'isCompany' => 0
        ]);
    }

    public static function getBruttoUser(): QUI\Interfaces\Users\User
    {
        try {
            return QUI::getUsers()->getUserByMail('netto-phpunit@pcsg.de');
        } catch (QUI\Exception) {
        }

        $SystemUser = QUI::getUsers()->getSystemUser();

        $User = QUI::getUsers()->createChild('netto-phpunit@pcsg.de', $SystemUser);
        $User->setAttribute('firstname', 'Gernot');
        $User->setAttribute('lastname', 'Vogel');
        $User->setAttribute('email', 'netto-phpunit@pcsg.de');
        $User->setAttribute('quiqqer.erp.isNettoUser', 0);
        $User->save($SystemUser);

        $Address = $User->getStandardAddress();
        $Address->setAttribute('firstname', 'Gernot (PHPUNIT)');
        $Address->setAttribute('lastname', 'Vogel');
        $Address->setAttribute('country', 'DE');
        $Address->setAttribute('street_no', 'Katrin-Herbst-Weg 3');
        $Address->setAttribute('zip', '56743');
        $Address->setAttribute('city', 'Herne');
        $Address->addMail('netto-phpunit@pcsg.de');
        $Address->addPhone([
            'no' => '0721 5303076',
            'type' => 'tel'
        ]);

        $Address->save($SystemUser);

        return $User;
    }

    public static function getRandomPayment(): QUI\ERP\Accounting\Payments\Types\Payment
    {
        $Payments = Payments::getInstance();
        $payments = $Payments->getPayments();

        /* @var $Payment QUI\ERP\Accounting\Payments\Types\Payment */
        $Payment = $payments[array_rand($payments)];
        $paymentType = $Payment->getAttribute('payment_type');

        if (!str_starts_with($paymentType, 'QUI\ERP\Accounting\Payments\Methods')) {
            return self::getRandomPayment();
        }

        return $Payment;
    }

    /**
     * @throws DoctrineException|QUI\ERP\Products\Product\Exception
     */
    public static function getRandomProduct(): QUI\ERP\Products\Product\Product
    {
        $connection = QUI::getDataBaseConnection();
        $table = QUI\ERP\Products\Utils\Tables::getProductTableName();

        $sqlCount = "SELECT COUNT(*) FROM $table";
        $totalRows = $connection->executeQuery($sqlCount)->fetchOne();
        $randomOffset = rand(0, $totalRows - 1);

        $sqlRandomEntry = "SELECT * FROM $table LIMIT 1 OFFSET $randomOffset";
        $randomEntry = $connection->executeQuery($sqlRandomEntry)->fetchAssociative();

        return QUI\ERP\Products\Handler\Products::getProduct($randomEntry['id']);
    }

    /**
     * @return QUI\ERP\Accounting\Article
     * @throws QUI\ERP\Products\Product\Exception
     * @throws QUI\Exception
     * @throws QUI\Users\Exception
     * @throws DoctrineException
     */
    public static function getRandomArticle(): QUI\ERP\Accounting\Article
    {
        $Product = ErpEntities::getRandomProduct();
        $BasketProduct = new QUI\ERP\Order\Basket\Product($Product->getId());
        $BasketProduct->setQuantity(1);

        return $BasketProduct->toArticle();
    }
}
